# -*- coding: utf-8 -*-

from rest_framework import serializers

from ... import models
from thx.api.serializers import (
    ThuxListSerializerMixin, ThuxRetrieveSerializerMixin, ThuxCreateSerializerMixin,
    ThuxUpdateSerializerMixin, ThuxPartialUpdateSerializerMixin, ThuxStatusSerializerMixin,
    ThuxEnableSerializerMixin, ThuxDisableSerializerMixin
)


# NOTIFICATION
class NotificationSerializer(serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification'
        model = models.Notification
        fields = '__all__'


class NotificationListSerializer(ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_list'
        model = models.Notification
        fields = '__all__'

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'backoffice':
            return fields
        if profile == 'customer':
            return fields
        return []


class NotificationRetrieveSerializer(ThuxRetrieveSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_retrieve'
        model = models.Notification
        fields = '__all__'

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'backoffice':
            return fields
        if profile == 'customer':
            return fields
        return []


class NotificationCreateSerializer(ThuxCreateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_create'
        model = models.Notification
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'backoffice':
            return fields
        if profile == 'customer':
            return fields
        return []

    def to_representation(self, instance):
        return NotificationListSerializer(instance, context=self.context).data


class NotificationUpdateSerializer(ThuxUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_update'
        model = models.Notification
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'backoffice':
            return fields
        if profile == 'customer':
            return fields
        return []

    def to_representation(self, instance):
        return NotificationRetrieveSerializer(instance, context=self.context).data


class NotificationPartialUpdateSerializer(ThuxPartialUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_partial_update'
        model = models.Notification
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'backoffice':
            return fields
        if profile == 'customer':
            return fields
        return []

    def to_representation(self, instance):
        return NotificationRetrieveSerializer(instance, context=self.context).data


class NotificationEnableSerializer(ThuxEnableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_enable'
        model = models.Notification
        fields = ()

    def update(self, instance, validated_data):
        instance.status = 1
        instance.save()
        return instance

    def to_representation(self, instance):
        return NotificationRetrieveSerializer(instance, context=self.context).data


class NotificationDisableSerializer(ThuxDisableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_disable'
        model = models.Notification
        fields = ()

    def update(self, instance, validated_data):
        instance.status = 0
        instance.save()
        return instance

    def to_representation(self, instance):
        return NotificationRetrieveSerializer(instance, context=self.context).data


# NOTIFICATION RECIPIENT
class NotificationRecipientSerializer(serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_recipient'
        model = models.NotificationRecipient
        fields = '__all__'


class NotificationRecipientListSerializer(ThuxListSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_recipient_list'
        model = models.NotificationRecipient
        fields = '__all__'

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'backoffice':
            return fields
        if profile == 'customer':
            return fields
        return []


class NotificationRecipientRetrieveSerializer(ThuxRetrieveSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_recipient_retrieve'
        model = models.NotificationRecipient
        fields = '__all__'

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'backoffice':
            return fields
        if profile == 'customer':
            return fields
        return []


class NotificationRecipientCreateSerializer(ThuxCreateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_recipient_create'
        model = models.NotificationRecipient
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'backoffice':
            return fields
        if profile == 'customer':
            return fields
        return []

    def to_representation(self, instance):
        return NotificationRecipientListSerializer(instance, context=self.context).data


class NotificationRecipientUpdateSerializer(ThuxUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_recipient_update'
        model = models.NotificationRecipient
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'backoffice':
            return fields
        if profile == 'customer':
            return fields
        return []

    def to_representation(self, instance):
        return NotificationRecipientRetrieveSerializer(instance, context=self.context).data


class NotificationRecipientPartialUpdateSerializer(ThuxPartialUpdateSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_recipient_partial_update'
        model = models.NotificationRecipient
        exclude = ('date_create', 'date_last_modify', 'creator', 'last_modifier')

    def get_field_names(self, *args, **kwargs):
        profile = self.context.get('view').kwargs['profile']
        fields = super().get_field_names(*args, **kwargs)
        if profile == 'superuser':
            return fields
        if profile == 'backoffice':
            return fields
        if profile == 'customer':
            return fields
        return []

    def to_representation(self, instance):
        return NotificationRecipientRetrieveSerializer(instance, context=self.context).data


class NotificationRecipientEnableSerializer(ThuxEnableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_recipient_enable'
        model = models.NotificationRecipient
        fields = ()

    def update(self, instance, validated_data):
        instance.status = 1
        instance.save()
        return instance

    def to_representation(self, instance):
        return NotificationRecipientRetrieveSerializer(instance, context=self.context).data


class NotificationRecipientDisableSerializer(ThuxDisableSerializerMixin, serializers.ModelSerializer):
    class Meta:
        ref_name = 'v1_notify_notification_recipient_disable'
        model = models.NotificationRecipient
        fields = ()

    def update(self, instance, validated_data):
        instance.status = 0
        instance.save()
        return instance

    def to_representation(self, instance):
        return NotificationRecipientRetrieveSerializer(instance, context=self.context).data
